@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2021, Mark Calabretta
+  WCSLIB 7.11 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2022, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -19,18 +19,23 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: tprj2.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: tprj2.c,v 7.11 2022/04/26 06:13:52 mcalabre Exp $
 *=============================================================================
 *
 * tproj2 tests projection routines by plotting test graticules using PGPLOT.
 *
 *---------------------------------------------------------------------------*/
 
-#include <cpgplot.h>
+// Needed to get nanosleep() from time.h.
+#define _POSIX_C_SOURCE 199309L
+
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
+
+#include <cpgplot.h>
 
 #include <prj.h>
 
@@ -460,6 +465,9 @@ struct prjprm *prj;
   yr[0] = 0.0f;
   cpgpt(1, xr, yr, 21);
 
+  // Allow 250ms to view the plot on fast machines.
+  struct timespec nano = {(time_t)0, 250000000L};
+  nanosleep(&nano, 0x0);
 
   cpgask(1);
   cpgpage();
