@@ -1,6 +1,6 @@
 /*============================================================================
-  WCSLIB 7.4 - an implementation of the FITS WCS standard.
-  Copyright (C) 1995-2021, Mark Calabretta
+  WCSLIB 7.11 - an implementation of the FITS WCS standard.
+  Copyright (C) 1995-2022, Mark Calabretta
 
   This file is part of WCSLIB.
 
@@ -19,17 +19,22 @@
 
   Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
   http://www.atnf.csiro.au/people/Mark.Calabretta
-  $Id: tspc.c,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+  $Id: tspc.c,v 7.11 2022/04/26 06:13:52 mcalabre Exp $
 *=============================================================================
 *
 * tspc tests the spectral transformation driver routines for closure.
 *
 *---------------------------------------------------------------------------*/
 
-#include <cpgplot.h>
+// Needed to get nanosleep() from time.h.
+#define _POSIX_C_SOURCE 199309L
+
 #include <math.h>
 #include <stdio.h>
 #include <string.h>
+#include <time.h>
+
+#include <cpgplot.h>
 
 #include <wcstrig.h>
 #include <spc.h>
@@ -49,38 +54,36 @@ double mars[7] = {4.5e5, 1.0, 27.0, 1.765, -1.077e6, 3.0, 5.0};
 int main()
 
 {
-  char text[80];
-  int  naxisj, nFail = 0, status;
-  double cdeltX, crpixj, crvalX, restfrq, restwav, x1, x2;
-
-
   printf(
     "Testing closure of WCSLIB spectral transformation routines (tspc.c)\n"
     "-------------------------------------------------------------------\n");
 
   // List status return messages.
   printf("\nList of spc status return values:\n");
-  for (status = 1; status <= 4; status++) {
+  for (int status = 1; status <= 4; status++) {
     printf("%4d: %s.\n", status, spc_errmsg[status]);
   }
 
 
   // PGPLOT initialization.
+  char text[80];
   strcpy(text, "/xwindow");
   cpgbeg(0, text, 1, 1);
 
-  naxisj = NSPEC;
-  crpixj = naxisj/2 + 1;
+  int    naxisj = NSPEC;
+  double crpixj = naxisj/2 + 1;
 
-  restfrq = 1420.40595e6;
-  restwav = C/restfrq;
-  x1 = 1.0e9;
-  x2 = 2.0e9;
-  cdeltX = (x2 - x1)/(naxisj - 1);
-  crvalX = x1 + (crpixj - 1.0)*cdeltX;
+  double restfrq = 1420.40595e6;
+  double restwav = C/restfrq;
+  double x1 = 1.0e9;
+  double x2 = 2.0e9;
+  double cdeltX = (x2 - x1)/(naxisj - 1);
+  double crvalX = x1 + (crpixj - 1.0)*cdeltX;
   printf("\nLinear frequency axis, span: %.1f to %.1f (GHz), step: %.3f "
          "(kHz)\n---------------------------------------------------------"
          "-----------------\n", x1*1e-9, x2*1e-9, cdeltX*1e-3);
+
+  int nFail = 0;
   nFail += closure("WAVE-F2W",     0.0,     0.0, naxisj, crpixj, cdeltX, crvalX);
   nFail += closure("VOPT-F2W",     0.0, restwav, naxisj, crpixj, cdeltX, crvalX);
   nFail += closure("ZOPT-F2W",     0.0, restwav, naxisj, crpixj, cdeltX, crvalX);
@@ -206,23 +209,20 @@ int closure (
   double crvalX)
 
 {
-  char ptype, sname[32], title[80], units[8], xtype, ylab[80];
-  int  nFail = 0, restreq, stat1[NSPEC], stat2[NSPEC], status;
-  register int j;
-  float  tmp, x[NSPEC], xmin, xmax, y[NSPEC], ymax, ymin;
-  double cdeltS, clos[NSPEC], crvalS, dSdX, resid, residmax, spec1[NSPEC],
-         spec2[NSPEC];
-  struct spcprm spc;
-
+  int status;
 
   // Get keyvalues for the required spectral axis type.
+  char   ptype, xtype;
+  int    restreq;
+  double crvalS, dSdX;
   if ((status = spcxps(ctypeS, crvalX, restfrq, restwav, &ptype, &xtype,
                        &restreq, &crvalS, &dSdX))) {
     printf("ERROR %d from spcxps() for %s.\n", status, ctypeS);
     return 1;
   }
-  cdeltS = cdeltX * dSdX;
+  double cdeltS = cdeltX * dSdX;
 
+  struct spcprm spc;
   spcini(&spc);
 
   if (ctypeS[5] == 'G') {
@@ -237,7 +237,8 @@ int closure (
   }
 
   // Construct the axis.
-  for (j = 0; j < naxisj; j++) {
+  double spec1[NSPEC];
+  for (int j = 0; j < naxisj; j++) {
     spec1[j] = (j+1 - crpixj)*cdeltS;
   }
 
@@ -255,19 +256,23 @@ int closure (
   strcpy(spc.code, ctypeS+5);
 
   // Convert the first to the second.
+  int    stat1[NSPEC], stat2[NSPEC];
+  double spec2[NSPEC];
   if ((status = spcx2s(&spc, naxisj, 1, 1, spec1, spec2, stat1))) {
     printf("spcx2s ERROR %d: %s.\n", status, spc_errmsg[status]);
   }
 
   // Convert the second back to the first.
+  double clos[NSPEC];
   if ((status = spcs2x(&spc, naxisj, 1, 1, spec2, clos, stat2))) {
     printf("spcs2x ERROR %d: %s.\n", status, spc_errmsg[status]);
   }
 
-  residmax = 0.0;
+  double residmax = 0.0;
 
   // Test closure.
-  for (j = 0; j < naxisj; j++) {
+  int nFail = 0;
+  for (int j = 0; j < naxisj; j++) {
     if (stat1[j]) {
       printf("%s: w =%20.12e -> %s = ???, stat = %d\n", ctypeS, spec1[j],
              spc.type, stat1[j]);
@@ -280,7 +285,7 @@ int closure (
       continue;
     }
 
-    resid = fabs((clos[j] - spec1[j])/cdeltS);
+    double resid = fabs((clos[j] - spec1[j])/cdeltS);
     if (resid > residmax) residmax = resid;
 
     if (resid > tol) {
@@ -298,20 +303,21 @@ int closure (
   cpgbbuf();
   cpgeras();
 
-  xmin = (float)(crvalS + spec1[0]);
-  xmax = (float)(crvalS + spec1[naxisj-1]);
-  ymin = (float)(spec2[0]) - xmin;
-  ymax = ymin;
-  for (j = 0; j < naxisj; j++) {
+  float x[NSPEC], y[NSPEC];
+  float xmin = (float)(crvalS + spec1[0]);
+  float xmax = (float)(crvalS + spec1[naxisj-1]);
+  float ymin = (float)(spec2[0]) - xmin;
+  float ymax = ymin;
+  for (int j = 0; j < naxisj; j++) {
     x[j] = (float)(j+1);
     y[j] = (float)(spec2[j] - (crvalS + spec1[j]));
     if (y[j] > ymax) ymax = y[j];
     if (y[j] < ymin) ymin = y[j];
   }
 
-  j = (int)crpixj + 1;
+  int j = (int)crpixj + 1;
   if (y[j] < 0.0) {
-    tmp  = ymin;
+    float tmp  = ymin;
     ymin = ymax;
     ymax = tmp;
   }
@@ -319,6 +325,7 @@ int closure (
   cpgask(0);
   cpgenv(1.0f, (float)naxisj, ymin, ymax, 0, -1);
 
+  char sname[32], title[80], units[8], ylab[80];
   cpgsci(1);
   cpgbox("ABNTS", 0.0f, 0, "BNTS", 0.0f, 0);
   spctyp(ctypeS, 0x0, 0x0, sname, units, 0x0, 0x0, 0x0);
@@ -338,6 +345,10 @@ int closure (
   cpgpt1((float)crpixj, 0.0f, 24);
   cpgebuf();
 
+  // Allow 200ms to view the plot on fast machines.
+  struct timespec nano = {(time_t)0, 200000000L};
+  nanosleep(&nano, 0x0);
+
   printf("Type <RETURN> for next page: ");
   (void)getchar();
 
