@@ -1,7 +1,7 @@
 *=======================================================================
 *
-* WCSLIB 7.4 - an implementation of the FITS WCS standard.
-* Copyright (C) 1995-2021, Mark Calabretta
+* WCSLIB 7.11 - an implementation of the FITS WCS standard.
+* Copyright (C) 1995-2022, Mark Calabretta
 *
 * This file is part of WCSLIB.
 *
@@ -20,37 +20,42 @@
 *
 * Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
 * http://www.atnf.csiro.au/people/Mark.Calabretta
-* $Id: twcsfix.f,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+* $Id: twcsfix.f,v 7.11 2022/04/26 06:13:52 mcalabre Exp $
 *=======================================================================
 
       PROGRAM TWCSFIX
 *-----------------------------------------------------------------------
 *
 * TWCSFIX tests the translation routines for non-standard WCS keyvalues,
-* the WCSFIX suite, and the spectral coordinate translation routine
-* WCSPTR.
+* the WCSFIX suite.  It also tests the change of celestial coordinate
+* system routine, WCSCCS, and the spectral coordinate translation
+* routine, WCSSPTR.
 *
 *-----------------------------------------------------------------------
       DOUBLE PRECISION TOL
       PARAMETER (TOL = 1D-10)
 
-*     CUNITia is set to arcsec below as an additional test.
+*     B1950.0 equatorial coordinates of the galactic pole.
+*     CUNITia is set to ARCSEC below as an additional test.
       DOUBLE PRECISION DEC, RA
-      PARAMETER (RA  = 265.62209470900D0 * 3600D0)
-      PARAMETER (DEC = -28.98849996030D0 * 3600D0)
+      PARAMETER (RA  = 192.2500D0 * 3600D0)
+      PARAMETER (DEC = +27.4000D0 * 3600D0)
 
 *     Number of axes.
       INTEGER   N
       PARAMETER (N = 3)
 
       INTEGER   I, J, NAXIS, VELREF
-      DOUBLE PRECISION BEPOCH, CDELT(N), CRPIX(N), CRVAL(N), MJDBEG,
-     :          OBSGEO_B, OBSGEO_H, OBSGEO_L, PC(N,N), RESTFRQ, RESTWAV
+      DOUBLE PRECISION BEPOCH, CDELT(N), CRPIX(N), CRVAL(N), EQUINOX,
+     :          IMGCRD(N), LATPOLE, LONPOLE, MJDBEG, OBSGEO_B, OBSGEO_H,
+     :          OBSGEO_L, PC(N,N), PHI, PIXCRD(N), RESTFRQ, RESTWAV,
+     :          THETA, WORLD(N)
       CHARACTER CTYPE(N)*72, CUNIT(N)*72, DATEAVG*72, DATEBEG*72,
      :          DATEEND*72, DATEOBS*72, SPECSYS*72
 
       COMMON /HEADER/ CRPIX, PC, CDELT, CRVAL, RESTFRQ, RESTWAV, BEPOCH,
-     :          MJDBEG, OBSGEO_L, OBSGEO_B, OBSGEO_H, NAXIS, VELREF
+     :          MJDBEG, OBSGEO_L, OBSGEO_B, OBSGEO_H, EQUINOX, NAXIS,
+     :          VELREF
       COMMON /HEADCH/ CTYPE, CUNIT, DATEOBS, DATEBEG, DATEAVG, DATEEND,
      :          SPECSYS
 
@@ -61,9 +66,10 @@
       INCLUDE 'wcsfix.inc'
       INTEGER   STAT(WCSFIX_NWCS), STATUS
       CHARACTER CTYPES*9
-      INTEGER   WCS(WCSLEN)
-      DOUBLE PRECISION DUMMY
-      EQUIVALENCE (WCS,DUMMY)
+      INTEGER   WCS0(WCSLEN), WCS1(WCSLEN)
+      DOUBLE PRECISION DUMMY0, DUMMY1
+      EQUIVALENCE (WCS0,DUMMY0)
+      EQUIVALENCE (WCS1,DUMMY1)
 
       DATA NAXIS   /N/
       DATA (CRPIX(J), J=1,N)
@@ -96,6 +102,8 @@
       DATA OBSGEO_B /-32.998406D0/
       DATA OBSGEO_H /   411.793D0/
 
+      DATA EQUINOX /1950.0D0/
+
 *     For testing SPCFIX.
       DATA VELREF  /2/
       DATA SPECSYS /'BARYCENT'/
@@ -107,11 +115,11 @@
      :        '-----------',/)
 
 *     This routine simulates the actions of a FITS header parser.
-      STATUS = WCSPTI (WCS, WCS_FLAG, -1, 0, 0)
-      CALL PARSER (WCS)
+      STATUS = WCSPTI (WCS0, WCS_FLAG, -1, 0, 0)
+      CALL PARSER (WCS0)
 
 *     Fix non-standard WCS keyvalues.
-      STATUS = WCSFIX (7, 0, WCS, STAT)
+      STATUS = WCSFIX (7, 0, WCS0, STAT)
       WRITE (*, 20) (STAT(I), I=1,WCSFIX_NWCS)
  20   FORMAT ('WCSFIX status returns: (',I2,6(',',I2),')',/)
 
@@ -122,36 +130,100 @@
       END IF
 
 *     Extract information from the FITS header.
-      STATUS = WCSSET (WCS)
+      STATUS = WCSSET (WCS0)
       IF (STATUS.NE.0) THEN
-        STATUS = WCSPERR(WCS, CHAR(0))
+        STATUS = WCSPERR(WCS0, CHAR(0))
+        GO TO 999
       END IF
 
       CALL FLUSH(6)
-      STATUS = WCSPRT (WCS)
+      STATUS = WCSPRT (WCS0)
       WRITE (*, 40)
  40   FORMAT (/,'------------------------------------',
      :          '------------------------------------')
 
-*     Should now have a 'VOPT-F2W' axis, translate it to frequency.
-      CTYPES = 'FREQ-???'
-      I = -1
-      STATUS = WCSSPTR (WCS, I, CTYPES)
+*     Make a copy of the wcsprm struct.
+      STATUS = WCSPTI (WCS1, WCS_FLAG, -1, 0, 0)
+      STATUS = WCSSUB (WCS0, -1, -1, WCS1)
       IF (STATUS.NE.0) THEN
-        STATUS = WCSPERR(WCS, CHAR(0))
+        STATUS = WCSPERR (WCS1, CHAR(0))
         GO TO 999
       END IF
 
-      STATUS = WCSSET (WCS)
+*     Transform equatorial B1950 to galactic coordinates.  The WCS has
+*     been constructed with the galactic pole coincident with the native
+*     pole of the projection in order to test the resolution of an
+*     indeterminacy.
+      STATUS = WCSCCS (WCS1, 123.0D0, 27.4D0, 192.25D0, 'GLON', 'GLAT',
+     :                 CHAR(0), 0D0, 'G')
       IF (STATUS.NE.0) THEN
-        STATUS = WCSPERR(WCS, CHAR(0))
+        STATUS = WCSPERR (WCS1, CHAR(0))
+        GO TO 999
+      END IF
+
+*     Should now have a 'VOPT-F2W' axis, translate it to frequency.
+      CTYPES = 'FREQ-???'
+      I = -1
+      STATUS = WCSSPTR (WCS1, I, CTYPES)
+      IF (STATUS.NE.0) THEN
+        STATUS = WCSPERR(WCS1, CHAR(0))
+        GO TO 999
+      END IF
+
+      STATUS = WCSSET (WCS1)
+      IF (STATUS.NE.0) THEN
+        STATUS = WCSPERR(WCS1, CHAR(0))
         GO TO 999
       END IF
 
       CALL FLUSH(6)
-      STATUS = WCSPRT (WCS)
+      STATUS = WCSPRT (WCS1)
 
-      STATUS = WCSFREE (WCS)
+*     Print before-and-afters.
+      WRITE (*, 50) (CRPIX(I), I=1,N)
+ 50   FORMAT (/,'Original and new coordinates of reference point ('
+     :        2(F4.1,', '),F3.1,'), lonpole, and latpole:')
+      STATUS = WCSGTD (WCS0, WCS_CRVAL,   CRVAL(1))
+      STATUS = WCSGTD (WCS0, WCS_LONPOLE, LONPOLE)
+      STATUS = WCSGTD (WCS0, WCS_LATPOLE, LATPOLE)
+      WRITE (*, 60) (CRVAL(I), I=1,N), LONPOLE, LATPOLE
+ 60   FORMAT (2(F14.6,', '),F14.2,', ',F14.6,', ',F14.6)
+
+      STATUS = WCSGTD (WCS1, WCS_CRVAL,   CRVAL(1))
+      STATUS = WCSGTD (WCS1, WCS_LONPOLE, LONPOLE)
+      STATUS = WCSGTD (WCS1, WCS_LATPOLE, LATPOLE)
+      WRITE (*, 60) (CRVAL(I), I=1,N), LONPOLE, LATPOLE
+
+*     Compute B1950 coordinates of a field point.
+      PIXCRD(1) = 1000D0
+      PIXCRD(2) = 1000D0
+      PIXCRD(3) =    1D0
+      WRITE (*, 70) (PIXCRD(I), I=1,N)
+ 70   FORMAT (/,'Original and new coordinates of field point (',
+     :        2(F6.1,', '),F3.1,'):')
+
+      STATUS = WCSP2S (WCS0, 1, N, PIXCRD, IMGCRD, PHI, THETA, WORLD,
+     :                 STAT)
+      IF (STATUS.NE.0) THEN
+        STATUS = WCSPERR (WCS0, CHAR(0))
+        GO TO 999
+      END IF
+
+      WRITE (*, 80) (WORLD(I), I=1,N)
+ 80   FORMAT (2(F14.6,', '),F14.2)
+
+*     Compute galactic coordinates of the same field point.
+      STATUS = WCSP2S (WCS1, 1, N, PIXCRD, IMGCRD, PHI, THETA, WORLD,
+     :                 STAT)
+      IF (STATUS.NE.0) THEN
+        STATUS = WCSPERR (WCS1, CHAR(0))
+        GO TO 999
+      END IF
+
+      WRITE (*, 80) (WORLD(I), I=1,N)
+
+      STATUS = WCSFREE (WCS0)
+      STATUS = WCSFREE (WCS1)
 
 
  999  CONTINUE
@@ -173,13 +245,15 @@
       PARAMETER (N = 3)
 
       INTEGER   I, J, NAXIS, STATUS, VELREF, WCS(*)
-      DOUBLE PRECISION BEPOCH, CDELT(N), CRPIX(N), CRVAL(N), MJDBEG,
-     :          OBSGEO_B, OBSGEO_H, OBSGEO_L, PC(N,N), RESTFRQ, RESTWAV
+      DOUBLE PRECISION BEPOCH, CDELT(N), CRPIX(N), CRVAL(N), EQUINOX,
+     :          MJDBEG, OBSGEO_B, OBSGEO_H, OBSGEO_L, PC(N,N), RESTFRQ,
+     :          RESTWAV
       CHARACTER CTYPE(N)*72, CUNIT(N)*72, DATEAVG*72, DATEBEG*72,
      :          DATEEND*72, DATEOBS*72, SPECSYS*72
 
       COMMON /HEADER/ CRPIX, PC, CDELT, CRVAL, RESTFRQ, RESTWAV, BEPOCH,
-     :          MJDBEG, OBSGEO_L, OBSGEO_B, OBSGEO_H, NAXIS, VELREF
+     :          MJDBEG, OBSGEO_L, OBSGEO_B, OBSGEO_H, EQUINOX, NAXIS,
+     :          VELREF
       COMMON /HEADCH/ CTYPE, CUNIT, DATEOBS, DATEBEG, DATEAVG, DATEEND,
      :          SPECSYS
 
@@ -220,6 +294,8 @@
       STATUS = WCSPTD (WCS, WCS_OBSGEO, OBSGEO_B, 5, 0)
       STATUS = WCSPTD (WCS, WCS_OBSGEO, OBSGEO_H, 6, 0)
 
+      STATUS = WCSPTD (WCS, WCS_EQUINOX, EQUINOX, 6, 0)
+
       STATUS = WCSPTC (WCS, WCS_SPECSYS, SPECSYS, 0, 0)
 
       RETURN
