@@ -1,7 +1,7 @@
 *=======================================================================
 *
-* PGSBOX 7.4 - draw curvilinear coordinate axes for PGPLOT.
-* Copyright (C) 1997-2021, Mark Calabretta
+* PGSBOX 7.11 - draw curvilinear coordinate axes for PGPLOT.
+* Copyright (C) 1997-2022, Mark Calabretta
 *
 * This file is part of PGSBOX.
 *
@@ -20,7 +20,7 @@
 *
 * Author: Mark Calabretta, Australia Telescope National Facility, CSIRO.
 * http://www.atnf.csiro.au/people/Mark.Calabretta
-* $Id: pgsbox.f,v 7.4 2021/01/31 02:24:52 mcalabre Exp $
+* $Id: pgsbox.f,v 7.11 2022/04/26 06:13:52 mcalabre Exp $
 *=======================================================================
 *
 * PGSBOX draws and labels a curvilinear coordinate grid.  The caller
@@ -1577,8 +1577,6 @@
       CHARACTER ESCAPE*1, EXPONT*20, FMT*8, IDENTS(3)*(*), TEXT*80,
      :          FTYPE(2)*1, TXT(2)*80
 
-      DATA ESCAPE /'\\'/
-
 *     These are to stop compiler messages about uninitialized variables.
       DATA DODEG, DOMIN, DOYEAR /3 * .FALSE./
       DATA XL, YL /2 * -999.0/
@@ -1992,6 +1990,9 @@
       MJD2(1) = -1D99
       MJD2(2) = -1D99
 
+*     Backslash.
+      ESCAPE = CHAR(92)
+
 *     Character height.
       CALL PGQCS (4, XCH, YCH)
       PRVEDG = 0
